#!/usr/bin/env python

from __future__ import division, print_function, absolute_import

import tempfile
import shutil
import pytest

import numpy as np
from numpy.testing import assert_equal

import pandas as pd
from pandas.util.testing import assert_frame_equal

import thorns as th


def square(x):
    return x**2

def multiply(x, y):
    return x*y


@pytest.fixture(scope="function")
def workdir(request):

    workdir = tempfile.mkdtemp()

    def fin():
        print("Removing temp dir: {}".format(workdir))

        shutil.rmtree(workdir, ignore_errors=True)

    request.addfinalizer(fin)

    return workdir



def test_map_serial(workdir):

    space = [{'x': i} for i in range(10)]

    results = th.util.map(
        square,
        space,
        backend='serial',
        cache='no',
        workdir=workdir,
    )


    expected = pd.DataFrame(
        {
            'x': range(10),
            0: np.arange(10)**2
        }
    ).set_index('x')

    assert_frame_equal(results, expected)






def test_map_cache(workdir):

    data = np.arange(10)
    dicts = [{'x':i} for i in data]

    df1 = th.util.map(
        square,
        dicts,
        backend='serial',
        cache='yes',
        workdir=workdir,
    )
    df2 = th.util.map(
        square,
        dicts,
        backend='serial',
        cache='yes',
        workdir=workdir,
    )


    assert_frame_equal(df1, df2)



def test_map_kwargs(workdir):

    space = [{'x': i} for i in range(10)]
    kwargs = {'y': 2}


    results = th.util.map(
        multiply,
        space,
        backend='serial',
        cache='no',
        workdir=workdir,
        kwargs=kwargs,
    )

    expected = pd.DataFrame(
        {
            'x': range(10),
            0: np.arange(10)*2
        }
    ).set_index('x')

    assert_frame_equal(results, expected)



def test_map_cache_with_kwargs(workdir):

    space = [{'x': i} for i in range(10)]

    th.util.map(
        multiply,
        space,
        backend='serial',
        cache='yes',
        workdir=workdir,
        kwargs={'y': 2},
    )

    # It should *not* recall old results, even though space is the
    # same.  It should calculate new results, because kwargs are not
    # the same.
    results = th.util.map(
        multiply,
        space,
        backend='serial',
        cache='yes',
        workdir=workdir,
        kwargs={'y': 3},
    )

    expected = pd.DataFrame(
        {
            'x': range(10),
            0: np.arange(10)*3
        }
    ).set_index('x')

    assert_frame_equal(results, expected)



def test_larger_dict_than_required_kwargs(workdir):

    space = [{
        'x': 1,
        'y': 2,
        'bla': 12,
    }]

    results = th.util.map(
        multiply,
        space,
        backend='serial',
        cache='no',
        workdir=workdir,
    )

    assert set(results.index.names) == set(space[0].keys())

    expected = pd.DataFrame({
        'x': [1],
        'y': [2],
        'bla': [12],
        0: [2],
    }).set_index(results.index.names)

    assert_frame_equal(results, expected)


def test_map_multiprocessing(workdir):

    space = [{'x': i} for i in range(10)]

    results = th.util.map(
        square,
        space,
        backend='multiprocessing',
        cache='no',
        workdir=workdir,
    )


    expected = pd.DataFrame(
        {
            'x': range(10),
            0: np.arange(10)**2
        }
    ).set_index('x')

    assert_frame_equal(results, expected)






@pytest.mark.skipif('True')
def test_map_serial_isolated(workdir):

    space = [{'x': i} for i in range(10)]

    results = th.util.map(
        square,
        space,
        backend='serial_isolated',
        cache='no',
        workdir=workdir,
    )


    expected = pd.DataFrame(
        {
            'x': range(10),
            0: np.arange(10)**2
        }
    ).set_index('x')

    assert_frame_equal(results, expected)




@pytest.mark.skipif('True')
def test_ipython_map(workdir):

    data = np.arange(10)
    dicts = [{'x':i} for i in data]

    results1 = th.util.map(
        square,
        dicts,
        backend='ipython',
        workdir=workdir
    )
    results2 = th.util.map(
        square,
        dicts,
        backend='ipython',
        workdir=workdir
    )

    assert_equal(
        data**2,
        list(results1)
    )
    assert_equal(
        data**2,
        list(results2)
    )




def test_cache(workdir):

    square_cached = th.util.cache(square, workdir=workdir)

    square_cached(x=2)

    result = square_cached(x=2)

    assert_equal(result, 4)





def test_dict_of_lists():

    dict_of_lists = {
        'x': [1,2,3],
        'y': [4,5]
    }

    actual = th.util.map(
        multiply,
        dict_of_lists,
        cache='no',
        backend='serial',
    )


    list_of_dicts = [
        {'x': 1, 'y': 4},
        {'x': 2, 'y': 4},
        {'x': 3, 'y': 4},
        {'x': 1, 'y': 5},
        {'x': 2, 'y': 5},
        {'x': 3, 'y': 5},
    ]


    expected = th.util.map(
        multiply,
        list_of_dicts,
        cache='no',
        backend='serial',
    )


    assert_frame_equal(
        actual,
        expected,
        check_like=True,
    )



if __name__ == '__main__':
    test_ipython_map()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test binary search.

"""

from __future__ import division, absolute_import, print_function

import numpy as np
from numpy.testing import assert_almost_equal

import thorns as th



def test_find_zero():


    def func(x, foo):
        y = 2*x
        return y


    x0 = th.util.find_zero(
        func=func,
        x1=-10,
        x2=10,
        kwargs={'foo': np.pi},
        xtol=1e-9
    )


    assert_almost_equal(x0, 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Plot a raster plot of auditory nerve spike trains generated using
Zilany et al. (2014) model.

"""

from __future__ import division, absolute_import, print_function

import thorns as th
from thorns.datasets import load_anf_zilany2014


def main():

    # Load spike trains
    spike_trains = load_anf_zilany2014()

    print(spike_trains.head())

    # Calculate vector strength
    cf, = spike_trains.cf.unique()
    onset = 10e-3

    trimmed = th.trim(spike_trains, onset, None)
    vs = th.vector_strength(trimmed, freq=cf)

    print()
    print("Vector strength: {}".format(vs))

    # Plot raster plot
    th.plot_raster(spike_trains)

    # Show the plot
    th.show()                   # Equivalent to plt.show()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from __future__ import division, print_function, absolute_import

import numpy as np
from numpy.testing import assert_equal, assert_almost_equal
import pytest

import pandas as pd

import thorns as th



def test_get_duration():

    trains = pd.DataFrame([
        {'duration': 12, 'spikes': np.arange(10)},
        {'duration': 12, 'spikes': np.arange(15)},
    ])

    duration = th.get_duration(trains)

    assert_equal(duration, 12)



def test_get_duration_error():

    trains = pd.DataFrame([
        {'duration': 12, 'spikes': np.arange(10)},
        {'duration': 123, 'spikes': np.arange(15)},
    ])

    with pytest.raises(ValueError):
        duration = th.get_duration(trains)




def test_firing_rate():

    trains = th.make_trains(
        [[0.1, 0.4],
         [0.4, 0.5, 0.6]],
        duration=1
    )

    rate = th.firing_rate(
        trains
    )

    assert_equal(rate, 2.5)



def test_correlation_index():

    trains = th.make_trains(
        [[1,3],
         [1,2,3]]
    )


    ci = th.correlation_index(
        trains,
        normalize=False
    )

    assert_equal(ci, 4)




def test_shuffled_autocorrelogram():

    trains = th.make_trains(
        [[1,3],
         [1,2,3]]
    )

    sac, bin_edges = th.shuffled_autocorrelogram(
        trains,
        coincidence_window=1,
        analysis_window=3,
        normalize=False
    )


    assert_equal(
        bin_edges,
        [-2., -1.,  0.,  1.,  2.,  3.]
    )
    assert_equal(
        sac,
        [2, 2, 4, 2, 2]
    )




def test_psth():

    trains = th.make_trains(
        [[0.5, 1.5, 2.5],
         [0.5, 2.5]]
    )

    psth, bin_edges = th.psth(
        trains,
        bin_size=1,
        normalize=False
    )


    assert_equal(
        psth,
        [2, 1, 2]
    )
    assert_equal(
        bin_edges,
        [0, 1, 2, 3]
    )



def test_psth_with_empty_trains():
    trains = th.make_trains(
        [[], []]
    )
    psth, bin_edges = th.psth(
        trains,
        bin_size=1,
        normalize=False
    )


    assert psth is None
    assert bin_edges is None


    ### duration != 0
    trains = th.make_trains(
        [[], []],
        duration=2
    )
    psth, bin_edges = th.psth(
        trains,
        bin_size=1,
        normalize=False
    )

    assert_equal(
        psth,
        [0,0]
    )
    assert_equal(
        bin_edges,
        [0,1,2]
    )


def test_spike_count():
    trains = th.make_trains(
        [[0.1, 0.2, 0.3], [0.1, 0.2, 0.3]],
        duration=[2, 4]
    )

    count = th.spike_count(trains)

    assert_equal(count, 6)




def test_isih():

    trains = th.make_trains(
        [[1,2,3], [2,5,8]]
    )

    isih, bin_edges = th.isih(
        trains,
        bin_size=1,
    )

    assert_equal(
        isih,
        [ 0, 2, 2]
    )
    assert_equal(
        bin_edges,
        [ 0, 1, 2, 3]
    )



def test_entrainment():

    trains = th.make_trains(
        [[1, 2, 3], [0, 2, 4]]
    )

    ent = th.entrainment(
        trains,
        freq=1,
        bin_size=0.1
    )

    assert_equal(ent, 0.5)




    ### NaN test
    trains = th.make_trains(
        [[1], []]
    )

    ent = th.entrainment(
        trains,
        freq=1,
        bin_size=0.1
    )

    assert np.isnan(ent)



def test_vector_strength():

    ### Uniform spikes
    trains = th.make_trains(
        [np.arange(0, 1, 1/3600)]
    )

    si = th.vector_strength(
        trains,
        freq=10
    )
    assert_almost_equal(si, 0)



    ### Perfect synchrony
    trains = th.make_trains(
        [np.zeros(100)],
        duration=0.1
    )
    si = th.vector_strength(
        trains,
        freq=10
    )
    assert_equal(si, 1)



    ### Carefully chosen
    trains = th.make_trains(
        [np.tile([0, 0.25], 10)]
    )

    si = th.vector_strength(
        trains,
        freq=1
    )

    assert_equal(si, np.sqrt(2)/2)



"""Utilities.

"""

from __future__ import division, print_function, absolute_import
from __future__ import unicode_literals

import os

from . dumpdb import dumpdb, loaddb, get_store, dump, load
from . maps import map, cache
from . bisection import find_zero

def mkdir(dirname):
    if not os.path.exists(dirname):
        os.makedirs(dirname)



"""Spike analysis software.

"""

from __future__ import division, print_function, absolute_import

__version__ = "1"

import os


if 'THpdb' in os.environ:
    import pdb
    import sys
    import traceback

    def info(type, value, tb):
        traceback.print_exception(type, value, tb)
        pdb.pm()
    sys.excepthook = info


if 'THlog' in os.environ:
    import logging

    if os.environ['THlog'] in ('d', 'debug'):
        level = 'DEBUG'

    else:
        level = os.environ['THlog'].upper()

    logger = logging.getLogger('thorns')
    logger.setLevel(level)


import thorns.util
import thorns.waves
import thorns.io

from thorns.spikes import (
    make_trains,
    trains_to_array,
    accumulate,
    select_trains,
    trim,
    fold,
)

from thorns.stats import (
    get_duration,
    shuffled_autocorrelogram,
    correlation_index,
    firing_rate,
    psth,
    isih,
    entrainment,
    vector_strength,
    spike_count,
    period_histogram
)

from thorns.plotting import (
    plot_raster,
    plot_psth,
    plot_isih,
    plot_neurogram,
    plot_sac,
    plot_period_histogram,
    plot_signal,
    show,
    gcf,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""This module implements permanent store for data.

"""
from __future__ import division, print_function, absolute_import
from __future__ import unicode_literals

import os
import datetime
import logging

from sys import version
import shelve
import collections
try:
    import cPickle as pickle
except ImportError:
    import pickle

import numpy as np
import pandas as pd

from math import isnan

logger = logging.getLogger('thorns')


class NDArrayHandler(object):
    @staticmethod
    def tag(a):
        return 'array'

    @staticmethod
    def rep(a):
        return a.tolist()

    @staticmethod
    def string_rep(a):
        return None


class NumpyIntHandler(object):
    @staticmethod
    def tag(i):
        return 'i'

    @staticmethod
    def rep(i):
        return int(i)

    @staticmethod
    def string_rep(i):
        return str(i)


class NumpyFloatHandler(object):
    @staticmethod
    def tag(f):
        return "z" if isnan(f) or f in (float('Inf'), float('-Inf')) else "d"

    @staticmethod
    def rep(f):
        if isnan(f):
            return "NaN"
        if f == float('Inf'):
            return "INF"
        if f == float("-Inf"):
            return "-INF"
        return float(f)

    @staticmethod
    def string_rep(f):
        return str(f)


def get_store(name='store', workdir='work'):
    """Return a quick and dirty shelve based persisten dict-like store.

    """
    fname = os.path.join(workdir, name + '.db')

    if not os.path.exists(workdir):
        os.makedirs(workdir)

    store = shelve.open(fname, protocol=-1)

    return store


def dump(data, name, workdir='work', backend='pickle'):
    """Dump a data object."""
    if not os.path.exists(workdir):
        os.makedirs(workdir)

    if backend == 'pickle':
        fname = os.path.join(workdir, name+'.pkl')

        with open(fname, 'w') as f:
            pickle.dump(data, f, -1)

    elif backend == 'transit':
        fname = os.path.join(workdir, name+'.json')

        from transit.writer import Writer

        with open(fname, 'w') as f:
            writer = Writer(f, 'json')

            writer.register(np.ndarray, NDArrayHandler)
            writer.register(np.int64, NumpyIntHandler)
            writer.register(np.float64, NumpyFloatHandler)

            writer.write(data)


def load(name, workdir='work', backend='pickle'):
    """Load a simple object from a store (no db)."""
    if not os.path.exists(workdir):
        os.makedirs(workdir)

    if backend == 'transit':
        fname = os.path.join(workdir, name+'.json')

        from transit.reader import Reader

        with open(fname, 'r') as f:
            reader = Reader('json')

            data = reader.read(f)

    else:
        raise NotImplementedError("load: {} backend not implemented".format(backend))

    return data


def dumpdb(data, name='dump', workdir='work', backend='hdf', kwargs=None):
    """Dump data in order to recall the most up-to-date records later.

    Parameters
    ----------
    data : pd.DataFrame
        Data that will be appended to the database.
    name : str, optional
        Base name of the pickle file.
    workdir : str, optional
        Directory for the data.
    backend : str, optional
        Data storage format.
    kwargs : dict, optional
        Additional parameters common for all data (MultiIndex will be
        extended).

    """
    if not os.path.exists(workdir):
        os.makedirs(workdir)

    if kwargs is not None:
        for k,v in kwargs.items():
            data[k] = v
        data = data.set_index(list(kwargs), append=True)

    if backend == 'hdf':
        fname = os.path.join(workdir, name+'.h5')

        logger.info("Dumping data into {}.".format(fname))

        now = datetime.datetime.now()
        key = now.strftime("T%Y%m%d_%H%M%S_%f")

        store = pd.io.pytables.HDFStore(fname, 'a')
        store[key] = data

        store.close()

    elif backend == 'shelve':
        fname = os.path.join(workdir, name+'.db')

        logger.info("Dumping data into {}.".format(fname))

        now = datetime.datetime.now()
        key = now.strftime("T%Y%m%d_%H%M%S_%f")

        store = shelve.open(fname, protocol=-1)

        store[key] = data

        store.close()

    elif backend == 'transit':
        from transit.writer import Writer

        logger.info("Dumping data into {}.".format(name))

        d = data.reset_index().to_dict('records')

        dump(d, name=name, workdir=workdir, backend='transit')

    else:
        raise NotImplementedError()



def loaddb(name='dump', workdir='work', backend='hdf', timestamp=False, load_all=False):
    """Recall dumped data discarding duplicated records.

    Parameters
    ----------
    name : str, optional
        Base of the data filename.
    workdir : str, optional
        Directory where the data is stored.
    backend : str, optional
        Backend used for data storage.
    timestamp : bool, optional
        Add an extra column with timestamps to the index.
    load_all : bool, optional
        If True, data from all experiments will be loaded from the
        dumpdb file.  The default is to load only the most recent
        data.

    Returns
    -------
    pd.DataFrame
        Data without duplicates.

    """
    if timestamp:
        raise NotImplementedError("Should add extra columnt with timestamps to the index of the output.")


    if backend == 'hdf':
        db = _loaddb_hdf(name, workdir, load_all)
    elif backend == 'shelve':
        db = _loaddb_shelve(name, workdir, load_all)
    else:
        raise NotImplementedError()

    return db



def _loaddb_hdf(name, workdir, load_all):

    fname = os.path.join(workdir, name+'.h5')
    store = pd.io.pytables.HDFStore(fname, 'r')

    logger.info("Loading data from {}".format(fname))

    if load_all:
        xkeys = collections.OrderedDict() # poor-man's ordered set
        dbs = []

        ### Get all tables from the store
        for t in sorted(store.keys()):
            df = store[t]

            # Just want ordered unique values in xkeys (ordered set would
            # simplify it: orderedset.update(df.index.names))
            for name in df.index.names:
                xkeys.setdefault(name)

            df = df.reset_index()
            dbs.append(df)

        db = pd.concat(dbs)

    else:
        last_key = sorted(store.keys())[-1]
        df = store[last_key]

        xkeys = df.index.names

        db = df.reset_index()

    store.close()

    ### Drop duplicates and set index
    db = db.drop_duplicates(
        subset=list(xkeys),
        keep='last',
    )

    db = db.set_index(list(xkeys))

    return db




def _loaddb_shelve(name, workdir, load_all):

    if load_all:
        raise NotImplementedError()

    fname = os.path.join(workdir, name+'.db')
    store = shelve.open(fname, protocol=-1)

    logger.info("Loading data from {}".format(fname))

    last_key = sorted(store.keys())[-1]

    df = store[last_key]

    store.close()

    # Drop records with duplicated keys
    xkeys = df.index.names

    db = df.reset_index()

    db = db.drop_duplicates(
        subset=list(xkeys),
        keep='last',
    )

    db = db.set_index(list(xkeys))

    return db



#!/usr/bin/env python

"""Binary search for threshold finding.

"""

from __future__ import division, print_function, absolute_import

import numpy as np


def find_zero(func, x1, x2, xtol=None, kwargs=None):
    """Find a zero crossing of a function using binary search.

    Parameters
    ----------
    func : function
        Function must be increasing and have one zero crossing.
    x1, x2 : float
        Lower and upper range.  Requirements (func(x1) < 0) and
        (func(x2) > 0).
    xtol : float, optional
        Range (x2-x1) at which the search stops.  If not specified,
        then 1e-3-th of the difference between x2 and x1 will be
        taken.
    kwargs : dict, optional
        Extra keyword arguments for the function.

    Returns
    -------
    float
        Function argument `x0` for which `func(x0) ~ 0`.

    """

    assert x1 < x2

    if kwargs is None:
        kwargs = {}

    if xtol is None:
        xtol = 1e-3 * np.abs(x2 - x1)

    if func(x1, **kwargs) > 0:
        return np.nan

    if func(x2, **kwargs) < 0:
        return np.nan

    while x2 - x1 > xtol:
        x = (x1 + x2) / 2
        y = func(x, **kwargs)

        if y > 0:
            x2 = x
        elif y < 0:
            x1 = x
        else:
            x1 = x2 = x

    x = (x1 + x2) / 2

    return x



#!/usr/bin/env python

from __future__ import division, print_function, absolute_import

import shutil
import pytest
import tempfile
import os

import numpy as np
from numpy.testing import assert_equal
import pandas as pd
from pandas.util.testing import assert_frame_equal

import thorns as th

import transit.reader


@pytest.fixture(scope="function")
def workdir(request):

    workdir = tempfile.mkdtemp()

    def fin():
        print("Removing temp dir: {}".format(workdir))

        shutil.rmtree(workdir, ignore_errors=True)

    request.addfinalizer(fin)

    return workdir




def test_dump_and_load_single_df(workdir):

    data = pd.DataFrame([
        {'a': 1, 'b': 1.1, 'c': 1   },
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
        {'a': 1, 'b': 1.1, 'c': 1.11}, # duplicate of the 0th row
    ]).set_index(['a', 'b'])

    expected = pd.DataFrame([
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
        {'a': 1, 'b': 1.1, 'c': 1.11},
    ]).set_index(['a', 'b'])


    th.util.dumpdb(data, workdir=workdir)

    actual = th.util.loaddb(workdir=workdir)

    assert_frame_equal(actual, expected)


def test_dump_and_load(workdir):

    data1 = pd.DataFrame([
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 55, 'y': 400, 'f': np.array([5,5])},
        {'x': 60, 'y': 400, 'f': np.array([2,3])},
    ]).set_index(['x','y'])

    data2 = pd.DataFrame([
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 60, 'y': 400, 'f': np.array([20,30])},
    ]).set_index(['x','y'])



    th.util.dumpdb(data1, workdir=workdir)
    th.util.dumpdb(data2, workdir=workdir)



    db = th.util.loaddb(workdir=workdir)


    expected = data2


    assert_frame_equal(db, expected)




def test_dump_and_load_all(workdir):

    data1 = pd.DataFrame([
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 55, 'y': 400, 'f': np.array([5,5])},
        {'x': 60, 'y': 400, 'f': np.array([2,3])},
    ]).set_index(['x','y'])

    data2 = pd.DataFrame([
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 60, 'y': 400, 'f': np.array([20,30])},
    ]).set_index(['x','y'])



    th.util.dumpdb(data1, workdir=workdir)
    th.util.dumpdb(data2, workdir=workdir)



    db = th.util.loaddb(workdir=workdir, load_all=True)


    expected = pd.DataFrame([
        {'x': 55, 'y': 400, 'f': np.array([5,5])},
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 60, 'y': 400, 'f': np.array([20,30])},
    ]).set_index(['x','y'])


    assert_frame_equal(db, expected)







def test_kwargs(workdir):

    data = pd.DataFrame([
        {'x': 50, 'y': 400, 'f': np.array([1,2])},
        {'x': 60, 'y': 400, 'f': np.array([2,3])},
    ]).set_index(['x','y'])



    th.util.dumpdb(
        data,
        kwargs={'type':'anf'},
        workdir=workdir,
    )


    db = th.util.loaddb(workdir=workdir)


    expected = pd.DataFrame([
        {'x': 50, 'y': 400, 'type': 'anf', 'f': np.array([1,2])},
        {'x': 60, 'y': 400, 'type': 'anf', 'f': np.array([2,3])},
    ]).set_index(['x','y','type'])


    assert_frame_equal(db, expected)




def test_dump_transit(workdir):

    data = pd.DataFrame([
        {'a': 1, 'b': 1.1, 'c': 1   },
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
    ]).set_index(['a', 'b'])

    th.util.dumpdb(data, workdir=workdir, backend='transit')

    reader = transit.reader.Reader('json')

    fname = os.path.join(workdir, 'dump.json')

    with open(fname, 'r') as f:
        actual = reader.read(f)

    desired = data.reset_index().to_dict('records')

    for a, d in zip(actual, desired):
        assert dict(a) == d



def test_dump_and_load_single_df_shelve(workdir):

    data = pd.DataFrame([
        {'a': 1, 'b': 1.1, 'c': 1   },
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
        {'a': 1, 'b': 1.1, 'c': 1.11}, # duplicate of the 0th row
    ]).set_index(['a', 'b'])

    expected = pd.DataFrame([
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
        {'a': 1, 'b': 1.1, 'c': 1.11},
    ]).set_index(['a', 'b'])


    th.util.dumpdb(data, workdir=workdir, backend='shelve')

    actual = th.util.loaddb(workdir=workdir, backend='shelve')

    assert_frame_equal(actual, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test thorns.waves module.

"""

from __future__ import division, absolute_import, print_function

import numpy as np
from numpy.testing import assert_equal

import thorns.waves as wv



def test_electrical_pulse_charge():

    durations = [1, 1, 1]
    amplitudes = [-1, 2, -1]

    fs = 100

    pulse = wv.electrical_pulse(
        fs=fs,
        amplitudes=amplitudes,
        durations=durations,
        charge=1
    )


    charge = np.sum(np.abs(pulse))/fs

    assert_equal(charge, 1)



def test_electrical_amplitudes_2():

    durations = [1, 0.5]

    amplitudes = wv.electrical_amplitudes(
        durations=durations,
        polarity=1,
    )


    assert_equal(amplitudes, [0.5, -1])


def test_electrical_amplitudes_3():

    durations = [0.5, 1, 0.5]
    ratio = 0.3
    polarity = 1

    amplitudes = wv.electrical_amplitudes(
        durations=durations,
        polarity=polarity,
        ratio=ratio
    )

    assert_equal(amplitudes, [0.3, -0.5, 0.7])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Demo that dumps and load data to a permanent storage.

"""

from __future__ import division, print_function, absolute_import

import pandas as pd
import thorns as th

def main():

    ### Create data with duplicates (records)
    data = pd.DataFrame([
        {'a': 1, 'b': 1.1, 'c': 1   },
        {'a': 2, 'b': 2.2, 'c': 4   },
        {'a': 3, 'b': 3.3, 'c': 9   },
        {'a': 1, 'b': 1.1, 'c': 1.11}, # duplicate of the 0th row
    ])

    data = data.set_index(['a', 'b'])


    ### Dump the data
    th.util.dumpdb(data)



    ### Load the data, note that duplicated resuts are dropped (only
    ### the most recent data is returned)
    db = th.util.loaddb()



    ### Show the results
    print(db)




if __name__ == "__main__":
    main()



#!/usr/bin/env python

from __future__ import division


raise NotImplementedError


import numpy as np

golden = 1.6180339887
pi = np.pi

# TODO: help, testdoc
def detect_peaks(fs, signal):
    d = np.diff(signal)
    s = np.sign(d)

    sd = np.diff(s)

    peak_idx = np.where(sd==-2)[0] + 1
    peak_val = signal[peak_idx]

    assert ((signal[peak_idx-1] < signal[peak_idx]).all() and
            (signal[peak_idx+1] < signal[peak_idx]).all())

    return peak_idx, peak_val



def root_mean_square(s):
    return np.sqrt( np.sum(s**2) / len(s) )

rms = root_mean_square






def make_time(signal, fs):
    """
    Return time vector for the signal.

    signal: signal
    fs: sampling frequency in Hz

    """
    tmax = (len(signal)-1) / fs
    return np.linspace(0, tmax, len(signal))

t = make_time



# TODO: allow values to be 0, change default values
def generate_ramped_tone(fs,
                         freq,
                         tone_duration=50e-3,
                         ramp_duration=2.5e-3,
                         pad_duration=55e-3,
                         dbspl=None):
    """ Generate ramped tone singal.

    fs: sampling frequency [Hz]
    freq: frequency of the tone [Hz]
    tone_durations: [s]
    ramp_duration: [s]
    pad_duration: [s]
    dbspl:

    """
    t = np.arange(0, tone_duration, 1/fs)
    s = np.sin(2 * np.pi * t * freq)
    if dbspl != None:
        s = set_dbspl(s, dbspl)

    ramp = np.linspace(0, 1, np.ceil(ramp_duration * fs))
    s[0:len(ramp)] = s[0:len(ramp)] * ramp
    s[-len(ramp):] = s[-len(ramp):] * ramp[::-1]

    pad = np.zeros(pad_duration * fs)
    s = np.concatenate( (s, pad) )

    return s




def now():
    from datetime import datetime
    t = datetime.now()

    now = "%04d%02d%02d-%02d%02d%02d" % (t.year,
                                         t.month,
                                         t.day,
                                         t.hour,
                                         t.minute,
                                         t.second)
    return now


def time_stamp(fname):
    from os import path

    root,ext = path.splitext(fname)
    return root + "__" + now() + ext

tstamp = time_stamp


def meta_stamp(fname, *meta_args, **meta_kwargs):
    """ Add meta data string to the file name.

    >>> meta_stamp('/tmp/a.txt', {'b':1, 'c':3}, d=12, e=13.6)
    '/tmp/a__c=3__b=1__e=13.6__d=12.txt'

    """
    from os import path

    root,ext = path.splitext(fname)
    meta_str = str()

    for meta_dict in meta_args:
        for meta_key in meta_dict:
            meta_str += _meta_sub_string(meta_key, meta_dict[meta_key])

    for meta_key in meta_kwargs:
        meta_str += _meta_sub_string(meta_key, meta_kwargs[meta_key])

    new_fname = root + meta_str + ext
    new_fname = new_fname.replace(' ', '_')
    return new_fname

mstamp = meta_stamp

def _meta_sub_string(var, value):
    return '__' + str(var) + '=' + str(value)



def generate_biphasic_pulse(fs,
                            pulse_width,
                            gap_width,
                            amplitude=1,
                            fstim=None,
                            stimulus_duration=None,
                            polarity='a'):
    """ Generate biphasic pulse for electrical stimulation.

    fs: Hz
    pulse_width: us
    gap_width: us
    amplitude: mA
    fstim: Hz
    stimulus_duration: ms
    polarity: 'c': cathodic, 'a': anodic

    """
    assert polarity in ('c', 'a')

    def idx(width):
        """ fs: Hz, width: us """
        return np.round(fs*width/1e6)

    if fstim is None:
        fstim = 1e6 / (2*pulse_width + gap_width) # Hz
    stim = np.zeros(np.round( fs/fstim ))

    stim[ 0:idx(pulse_width) ] = amplitude
    stim[ idx(pulse_width+gap_width):idx(2*pulse_width+gap_width) ] = -amplitude

    if stimulus_duration is not None:
        times = np.ceil(stimulus_duration / (len(stim)*1000/fs))
        stim = np.tile(stim, times)
        stim = stim[0:np.ceil(stimulus_duration*fs/1000)]

    if polarity == 'c':
        stim = -stim

    return stim


def plot_signal(fs, s):
    import biggles

    t = make_time(fs, s)

    plot = biggles.FramedPlot()
    plot.add( biggles.Current(t, s) )
    plot.xlabel = Time (ms)

    return plot



if __name__ == "__main__":
    import doctest

    print "Doctest start:"
    doctest.testmod()
    print "done."
